/*****************************************************************************/
/*                                                                           */
/*     Licensed Materials - Property of IBM                                  */
/*                                                                           */
/*     5622-078                                                              */
/*     (C) Copyright IBM Corp. 1988, 1995 All Rights Reserved.               */
/*                                                                           */
/*     US Government Users Restricted Rights - Use, duplication or           */
/*     disclosure restricted by GSA ADP Schedule Contract with               */
/*     IBM Corp.                                                             */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*                         Connection Manager API                            */
/*                                                                           */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   CMI Symbol and Structure definitions used by CM.                        */
/*                                                                           */
/*****************************************************************************/





/*****************************************************************************/
/*                                                                           */
/*                         CMI Includes and Defines                          */
/*                                                                           */
/*****************************************************************************/
#ifndef _CMI_CM_H_32
#define _CMI_CM_H_32

#ifndef VOID
  typedef void               VOID;
#endif

#ifndef _32_INCL
  typedef unsigned short int WORD;
  typedef unsigned long int  DWORD;
  #define _32_INCL
#endif

#ifndef _32_INCL
  #include <wcl1632.h>
#endif

#ifndef _SF_H_32
#include "sf.h"
#define _SF_H_32
#endif



/*****************************************************************************/
/*                                                                           */
/*                         CMI Function Prototype                            */
/*                                                                           */
/*****************************************************************************/

ULONG APIENTRY CMI( PVOID );



/*****************************************************************************/
/*                                                                           */
/*                            Common Status                                  */
/*                                                                           */
/*****************************************************************************/

// Common Status Control Block
// ---------------------------
typedef struct CMIComStatus
{
     WORD   CompCodeCategory;
     WORD   ReportLocation;
     DWORD  ReportCompCode;
     WORD   Reserved;
     WORD   OrigLocation;
     DWORD  OrigCompCode;
     DWORD  SysLogID;
} CMI_COM_COMSTATUS;

// CompCodeCatagory
// ----------------
#define CMI_CM_CMP_CAT_REQ_SUCCESSFUL   0x00
#define CMI_CM_CMP_CAT_REQ_REJECTED     0x08
#define CMI_CM_CMP_CAT_REQ_ERROR        0x10
#define CMI_CM_CMP_CAT_STATE_ERROR      0x20
#define CMI_CM_CMP_CAT_USAGE_ERROR      0x40
#define CMI_CM_CMP_CAT_PERM_ERROR       0x80


// ReportLocation & OrigLocation
// -----------------------------
#define CMI_CM_LOC_LOCAL                0x10
#define CMI_CM_LOC_PATH                 0x20
#define CMI_CM_LOC_LINE                 0x20
#define CMI_CM_LOC_REMOTE               0x30

// ReportCompCode
// ---------------
#define CMI_CM_LAYER_ID_CM              0x00
#define CMI_CM_LAYER_ID_DM              0x00
#define CMI_CM_LAYER_ID_PCM             0x02
#define CMI_CM_LAYER_ID_DLC             0x06
#define CMI_CM_LAYER_ID_MAC             0x0A


/*****************************************************************************/
/* Common Primitive Control Information Defines                              */
/*                                                                           */
/*****************************************************************************/

// Identifier Type
// ---------------
#define CMI_CM_ID_TYPE_NONE     0x00
#define CMI_CM_ID_TYPE_U_CI_ID  0x01
#define CMI_CM_ID_TYPE_U_CE_ID  0x01
#define CMI_CM_ID_TYPE_U_DEV_ID 0x01
#define CMI_CM_ID_TYPE_P_CI_ID  0x02
#define CMI_CM_ID_TYPE_P_CE_ID  0x02
#define CMI_CM_ID_TYPE_P_DEV_ID 0x02
#define CMI_CM_ID_TYPE_U_SAP_ID 0x03
#define CMI_CM_ID_TYPE_P_SAP_ID 0x04

// Subfield Identifier
// -------------------
#define CMI_CM_SFID_PRM_SPECIFIC        0x04

// Subfield Type
// -------------
#define CMI_CM_SFTYPE_PRM_SPECIFIC      0xFF
#define CMI_CM_SFTYPE_MON_INFO          0xFE

// Null character - used for unspecified Directory Entry Name,
// Incoming Call Routing Table Entry Name or PCM Name
// ----------------------------------------------------------
#define NULL_CHAR                       0x00

// Value not provided - used for all one byte fields for which
// the value is not provided
// -----------------------------------------------------------
#define VALUE_NOT_PROVIDED              0xFF



/*****************************************************************************/
/*                                                                           */
/*                   CM_ACTIVATE_SAP Request Primitive                       */
/*                                                                           */
/*****************************************************************************/
// Control Information for CM_ACTIVATE_SAP Request
// -----------------------------------------------
typedef struct CI_CM_Act_SAP_Req
{
     WORD   CtrlInfoLength;     // sizeof(CI_CM_ACT_SAP_REQ) struct
     WORD   PrimitiveCode;      // CMI_CM_PC_ACT_SAP_REQ
     DWORD  DataInfoLength;     // 0
     WORD   Selector;           // 0
     WORD   IDType;             // 0
     DWORD  Reserved0;          // changed to reserved
     DWORD  U_SAP_ID;           // Appl assigned SAP identifier
     WORD   Reserved1;          // 0
     WORD   ServiceType;        // 0
     DWORD  Reserved2;          // Combined to a rsvd word
     WORD   (* APIENTRY ApplEP )( PVOID );
     BYTE   Reserved3[64];      // 0
} CI_CM_ACT_SAP_REQ;

// Primitive Code for CMI_ACTIVATE_SAP Request
// -------------------------------------------
#define CMI_CM_PC_ACT_SAP_REQ           0x0F0D


/*****************************************************************************/
/*                                                                           */
/*                   CM_ACTIVATE_SAP Confirm Primitive                       */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_ACTIVATE_SAP Confirm
// -----------------------------------------------
typedef struct CI_CM_Act_SAP_Cnf
{
     WORD               CtrlInfoLength; // sizeof(CI_CM_ACT_SAP_CNF)
     WORD               PrimitiveCode;  // CMI_CM_PC_ACT_SAP_CNF
     DWORD              DataInfoLength; // Total length of Data Area
     WORD               Selector;       // 0
     WORD               IDType;         // CMI_CM_ID_TYPE_U_SAP_ID
     DWORD              U_SAP_ID;       // Appl assigned SAP identifier
     CMI_COM_COMSTATUS  CommonStatus;   // Common Status sub structure
     DWORD              P_SAP_ID;       // CM assigned SAP identifier
     BYTE               Reserved1[32];  // 0
} CI_CM_ACT_SAP_CNF;

// Primitive Code for CMI_ACTIVATE_SAP Confirm
// -------------------------------------------
#define CMI_CM_PC_ACT_SAP_CNF           0xCF0D



// Primitive Specific Subfield for CM_ACTIVATE_SAP Confirm
// subarray of local number information
// -------------------------------------------------------
typedef struct CM_LocalNumInfo
{
     WORD  NumberType;     //
     WORD  NumberPlan;     //
     WORD  NumberLength;   // Number of digits
     WORD  Pad_1;          // Alignment
     BYTE  Pad_2[3];       // Alignment
     BYTE  NumberDigits[1];// IA5 digits
} CM_LOCALNUMINFO;

// Number Type
// -----------
#define SF_NUMBER_TYPE_UNSPECIFIED      0x00
#define SF_NUMBER_TYPE_INTERNATIONAL    0x01
#define SF_NUMBER_TYPE_NATIONAL         0x02
#define SF_NUMBER_TYPE_NETWORK_SPECIFIC 0x03
#define SF_NUMBER_TYPE_SUBSCRIBER       0x04
#define SF_NUMBER_TYPE_ABBREVIATED      0x06

// Numbering Plan
// --------------
#define SF_NUMBER_PLAN_UNKNOWN          0x00
#define SF_NUMBER_PLAN_ISDN             0x01
#define SF_NUMBER_PLAN_PRIVATE          0x09


// Channel Information Subfield
// ----------------------------
typedef struct CM_ChannelInfo
{
     DWORD  MacAddr;       // Reserved
     BYTE   PermName[32];  // Permanent Connect Name
     WORD   Sync;          // Channel Synchronicity
     WORD   Rsvd;          //
     WORD   Modem;         // modem Class
     WORD   pad_1;         // Alignment
} CM_CHANNELINFO;

// Channel Synchronicity
// ---------------------
#define CHANNEL_UNSPECIFIED             0x00
#define CHANNEL_ASYNCHRONOUS            0x01
#define CHANNEL_SYNCHRONOUS             0x02
#define CHANNEL_AUTOSYNC                0x04

// Modem Class
// -----------
#define MODEM_CLASS_UNSPECIFIED         0x0000
#define MODEM_CLASS_V22                 0x0001
#define MODEM_CLASS_V22BIS              0x0002
#define MODEM_CLASS_V32                 0x0003
#define MODEM_CLASS_V32BIS              0x0004
#define MODEM_CLASS_PROPRIETARY         0x00FE


// CM ActivateSAP Confirm Primitive Specific Subfield
// --------------------------------------------------
typedef struct SF_CM_Act_Sap_Cnf
{
     WORD             SFLength;       // sizeof(SF_CM_ACT_SAP_CNF)
     BYTE             SFIdentifier;   // CMI_CM_SFID_PRM_SPECIFIC
     BYTE             SFType;         // CMI_CM_SF_PCM_INFORMATION
     BYTE             ModuleName[16]; // ASCIIZ name of module instance
     WORD             NetworkType;    // GCCI_COM_NET_TYPE_*
     WORD             PCMState;       // PCM State
     WORD             LocalNumCount;  // Number of local Numbers
     WORD             BChannelsDef;   // Number of defined B channels
     WORD             MaxNumChan;     // Max # of channels - use by deep
     WORD             Pad_1;          // Aligment
     CM_CHANNELINFO   ChanInfoArr[1]; // Array of channel info.s
     CM_LOCALNUMINFO  LocalNumArr[1]; // Array of Local Numbers
} SF_CM_ACT_SAP_CNF;

// Subfield Type
// -------------
#define CMI_CM_SF_PCM_INFORMATION       0x01

// NetworkType
// -----------
#define CMI_CM_NET_TYPE_ISDN            0x01
#define CMI_CM_NET_TYPE_VSERIES         0x02
#define CMI_CM_NET_TYPE_ISDNX           0x04
#define CMI_CM_NET_TYPE_X25             0x08
#define CMI_CM_NET_TYPE_SDLC            0x20
#define CMI_CM_NET_TYPE_FRAME_RELAY     0x80
#define CMI_CM_NET_TYPE_DA_CHANNEL      0x81
#define CMI_CM_NET_TYPE_GENERIC         0x40

// PCM State
// ---------
#define CMI_CM_PCM_STATE_NOT_STARTED    0x0000
#define CMI_CM_PCM_STATE_ACTIVATING     0x0001
#define CMI_CM_PCM_STATE_ACTIVE         0x0002
#define CMI_CM_PCM_STATE_DEACTIVATING   0x0004
#define CMI_CM_PCM_STATE_DEACTIVATED    0x0008





/*****************************************************************************/
/*                                                                           */
/*                   CM_DEACTIVATE_SAP Request Primitive                     */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_DEACTIVATE_SAP Request
// -------------------------------------------------
typedef struct CI_CM_Deact_SAP_Req
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_DEACT_SAP_REQ)
     WORD   PrimitiveCode;  // CMI_CM_PC_DEACT_SAP_REQ
     DWORD  DataInfoLength; // 0
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_P_SAP_ID
     DWORD  P_SAP_ID;       // CM assigned SAP identifier
     BYTE   Reserved1[64];  // 0
} CI_CM_DEACT_SAP_REQ;

// Primitive Code for CMI_DEACTIVATE_SAP Request
// ---------------------------------------------
#define CMI_CM_PC_DEACT_SAP_REQ         0x0F0E



/*****************************************************************************/
/*                                                                           */
/*                   CM_DEACTIVATE_SAP Confirm Primitive                     */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_DEACTIVATE_SAP Confirm
// -------------------------------------------------
typedef struct CI_CM_Deact_SAP_Cnf
{
     WORD               CtrlInfoLength; // sizeof(CI_CM_DEACT_SAP_CNF)
     WORD               PrimitiveCode;  // CMI_CM_PC_DEACT_SAP_CNF
     DWORD              DataInfoLength; // 0
     WORD               Selector;       // 0
     WORD               IDType;         // CMI_CM_ID_TYPE_U_SAP_ID
     DWORD              U_SAP_ID;       // Appl assigned SAP identifier
     CMI_COM_COMSTATUS  CommonStatus;   // Common Status sub structure
} CI_CM_DEACT_SAP_CNF;

// Primitive Code for CMI_DEACTIVATE_SAP Confirm
// ----------------------------------------------
#define CMI_CM_PC_DEACT_SAP_CNF         0xCF0E



/*****************************************************************************/
/*                                                                           */
/*                   CM_DEACTIVATE_SAP Indication Primitive                  */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_DEACTIVATE_SAP Indicate
// --------------------------------------------------
typedef struct CI_CM_Deact_SAP_Ind
{
     WORD               CtrlInfoLength; // sizeof(CI_CM_DEACT_SAP_IND)
     WORD               PrimitiveCode;  // CMI_CM_PC_DEACT_SAP_IND
     DWORD              DataInfoLength; // 0
     WORD               Selector;       // 0
     WORD               IDType;         // CMI_CM_ID_TYPE_U_SAP_ID
     DWORD              U_SAP_ID;       // Appl assigned SAP identifier
     CMI_COM_COMSTATUS  CommonStatus;   // Common Status sub structure
} CI_CM_DEACT_SAP_IND;

// Primitive Code for CMI_DEACTIVATE_SAP Indicate
// ----------------------------------------------
#define CMI_CM_PC_DEACT_SAP_IND         0x4F0E


/*****************************************************************************/
/*                                                                           */
/*                   CM_CALL_ESTABLISH Request Primitive                     */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_ESTABLISH Request
// -------------------------------------------------
typedef struct CI_CM_Call_Estab_Req
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_CALL_ESTAB_REQ)
     WORD   PrimitiveCode;  // CMI_CM_PC_CALL_ESTAB_REQ
     DWORD  DataInfoLength; // length of data area
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_P_SAP_ID
     DWORD  P_SAP_ID;       // CM assigned SAP identifier
     DWORD  U_CI_ID;        // Appl assigned Call identifier
     DWORD  pad_1;          // Alignment
} CI_CM_CALL_ESTAB_REQ;

// Primitive Code for CM_CALL_ESTABLISH Request
// --------------------------------------------
#define CMI_CM_PC_CALL_ESTAB_REQ        0x0F10


// Primitive Specific Subfield for CM_CALL_ESTABLISH Request
// ---------------------------------------------------------
typedef struct SF_CM_Call_Estab_Req
{
     WORD   SFLength;           // sizeof(SF_CM_CALL_ESTAB_REQ)
     BYTE   SFIdentifier;       // CMI_CM_SFID_PRM_SPECIFIC
     BYTE   SFType;             // CMI_CM_SFTYPE_PRM_SPECIFIC
     BYTE   SFEntryName[16];    // Outgoing call directory entry id
     BYTE   SFPcmName[16];      // Name of PCM to setup call through
     WORD   SFPcmType;          //
     WORD   SFProtId;           //
     WORD   SFLogAdptnum;       // Appl specified logical adapter #
     WORD   SFSessMode;         // Session Direction
     BYTE   SFPermConnName[32]; // perm conn name
} SF_CM_CALL_ESTAB_REQ;

// PCM type
// --------
#define CMI_CM_PCM_TYPE_UNSPECIFIED     0x10
#define CMI_CM_PCM_TYPE_ISDN            0x01

// Protocol Identifier
// -------------------
#define CMI_CM_PROT_ID_UNSPECIFIED      0x00
#define CMI_CM_PROT_ID_LAN              0x01
#define CMI_CM_PROT_ID_X_25             0x03
#define CMI_CM_PROT_ID_SDLC             0x04
#define CMI_CM_PROT_ID_IDLC             0x05
#define CMI_CM_PROT_ID_NULL             0x06

// Session Mode
// ------------
#define CMI_CM_SESS_UNSPECIFIED         0x00
#define CMI_CM_SESS_OUTGOING            0x01
#define CMI_CM_SESS_INCOMING            0x02



/*****************************************************************************/
/*                                                                           */
/*                   CM_CALL_ESTABLISH Confirm Primitive                     */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_ESTABLISH Confirm
// -------------------------------------------------
typedef struct CI_CM_Call_Estab_Cnf
{
     WORD               CtrlInfoLength; // sizeof(CI_CM_CALL_ESTAB_CNF)
     WORD               PrimitiveCode;  // CMI_CM_PC_CALL_ESTAB_CNF
     DWORD              DataInfoLength; // size of Data Information Area
     WORD               Selector;       // 0
     WORD               IDType;         // CMI_CM_ID_TYPE_U_CI_ID
     DWORD              U_CI_ID;        // Appl assigned Call Id
     CMI_COM_COMSTATUS  CommonStatus;   //
     DWORD              P_CI_ID;        // CM assigned Call Id
     DWORD              pad_1;          // Alignment
} CI_CM_CALL_ESTAB_CNF;

// Primitive Code for CM_CALL_ESTABLISH Confirm
// --------------------------------------------
#define CMI_CM_PC_CALL_ESTAB_CNF        0xCF10


// Primitive Specific Subfield for CM_CALL_ESTABLISH Confirm
// ---------------------------------------------------------
typedef struct SF_CM_Call_Estab_Cnf
{
     WORD   SFLength;       // sizeof(SF_CM_CALL_ESTAB_CNF)
     BYTE   SFIdentifier;   // CMI_CM_SFID_PRM_SPECIFIC
     BYTE   SFType;         // CMI_CM_SFTYPE_PRM_SPECIFIC
     BYTE   SFReserved1[16];// 0
     BYTE   SFPcmName[16];  // Name of PCM call setup through
     WORD   SFPcmType;      // Type of PCM call setup through
     WORD   SFPad_1;        // Alignment
} SF_CM_CALL_ESTAB_CNF;

// PCM type
// --------
#define CMI_CM_PCM_TYPE_ISDN            0x01


/*****************************************************************************/
/*                                                                           */
/*                  CM_CALL_ESTABLISH Indication Primitive                   */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_ESTABLISH Indication
// ----------------------------------------------------
typedef struct CI_CM_Call_Estab_Ind
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_CALL_ESTAB_IND)
     WORD   PrimitiveCode;  // CMI_CM_PC_CALL_ESTAB_IND
     DWORD  DataInfoLength; // length of Data Information Area
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_U_CE_ID
     DWORD  U_CE_ID;        // Appl assigned Call Enablement Id
     DWORD  P_CI_ID;        // CM assigned Call Id
     DWORD  Pad_1;          // Alignment
} CI_CM_CALL_ESTAB_IND;

// Primitive Code for CM_CALL_ESTABLISH Indication
// -----------------------------------------------
#define CMI_CM_PC_CALL_ESTAB_IND        0x4F10


// Primitive Specific Subfield for CM_CALL_ESTABLISH Indication
// ------------------------------------------------------------
typedef struct SF_CM_Call_Estab_Ind
{
     WORD   SFLength;       // sizeof(SF_CM_CALL_ESTAB_CNF)
     BYTE   SFIdentifier;   // CMI_CM_SFID_PRM_SPECIFIC
     BYTE   SFType;         // CMI_CM_SFTYPE_PRM_SPECIFIC
     BYTE   SFReserved1[16];// 0
     BYTE   SFPcmName[16];  // Name of PCM call setup through
     WORD   SFPcmType;      // Type of PCM call setup through
     WORD   SFPad_1;        // Alignment
} SF_CM_CALL_ESTAB_IND;

// PCM type
// --------
#define CMI_CM_PCM_TYPE_ISDN            0x01


// Primitive Specific Subfield for CM_MON_CALL_ESTABLISH Indication
// ----------------------------------------------------------------
typedef struct SF_CM_Mon_Call_Estab_Ind
{
     WORD   SFLength;         // sizeof(SF_CM_CALL_ESTAB_CNF)
     BYTE   SFIdentifier;     // CMI_CM_SFID_PRM_SPECIFIC
     BYTE   SFType;           // CMI_CM_SFTYPE_MON_INFO
     WORD   SFPcmType;        // Type of PCM call setup through
     WORD   SFPad_1;          // Alignment
     BYTE   SFPcmName[16];    // PCM Name
     BYTE   SFDirEntName[16]; // Dir Entry Name
     WORD   SFModemType;      // Async, Sync, or unspecified
     WORD   SFPhoneNumLen;    // Phone number length
     BYTE   SFPad_2[3];       // Alignment
     BYTE   SFPhoneNum[1];    // Phone number
} SF_CM_MON_CALL_ESTAB_IND;

// Service Type
// ------------
#define  CMI_CM_SRV_TYPE_MONITOR_APPL  0x10



/*****************************************************************************/
/*                                                                           */
/*                   CM_CALL_ESTABLISH Response Primitive                        */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_ESTABLISH Response
// --------------------------------------------------
typedef struct CI_CM_Call_Estab_Rsp
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_CALL_ESTAB_RSP)
     WORD   PrimitiveCode;  // CMI_CM_PC_CALL_ESTAB_RSP
     DWORD  DataInfoLength; // length of Data Information Area
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_P_CI_ID
     DWORD  P_CI_ID;        // CM assigned Call Id
     DWORD  U_CI_ID;        // Appl assigned Call Id
     WORD   ActionCode;     // Action Code
     WORD   Pad_1;          // Alignment
} CI_CM_CALL_ESTAB_RSP;

// Primitive Code for CM_CALL_ESTABLISH Response
// ---------------------------------------------
#define CMI_CM_PC_CALL_ESTAB_RSP        0x8F10

// Action Code defines
// -------------------
#define CMI_CM_ACT_CODE_ACCEPT_CALL    0x0000
#define CMI_CM_ACT_CODE_REJECT_CALL    0x0001
#define CMI_CM_ACT_CODE_PENDING        0x0002


// Primitive Specific Subfield for CM_CALL_ESTABLISH Response
// ----------------------------------------------------------
typedef struct SF_CM_Call_Estab_Rsp
{
     WORD   SFLength;       // sizeof(SF_CM_CALL_ESTAB_RSP)
     BYTE   SFIdentifier;   // CMI_CM_SFID_PRM_SPECIFIC
     BYTE   SFType;         // CMI_CM_SFTYPE_PRM_SPECIFIC
     WORD   SFProtId;       //
     WORD   SFLogAdptnum;   // Appl specified logical adapter #
     DWORD  SFPad_1;        // Alignment
} SF_CM_CALL_ESTAB_RSP;

// Protocol Identifier
// -------------------
#define CMI_CM_PROT_ID_UNSPECIFIED      0x00
#define CMI_CM_PROT_ID_LAN              0x01
#define CMI_CM_PROT_ID_X_25             0x03
#define CMI_CM_PROT_ID_SDLC             0x04
#define CMI_CM_PROT_ID_IDLC             0x05
#define CMI_CM_PROT_ID_NULL             0x06



/*****************************************************************************/
/*                                                                           */
/*                   CM_CALL_CLEAR Request Primitive                         */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_CLEAR Request
// ---------------------------------------------
typedef struct CI_CM_Call_Clear_Req
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_CALL_CLEAR_REQ)
     WORD   PrimitiveCode;  // CMI_CM_PC_CALL_CLEAR_REQ
     DWORD  DataInfoLength; // length of Data Information Area
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_P_CI_ID OR
                            // CMI_CM_ID_TYPE_P_SAP_ID
     DWORD  Identifier;     // CM assigned Call or SAP identifier
     DWORD  U_CI_ID;        //
     DWORD  Pad_1;          // Alignment
} CI_CM_CALL_CLEAR_REQ;

// Primitive Code for CM_CALL_CLEAR Request
// ----------------------------------------
#define CMI_CM_PC_CALL_CLEAR_REQ        0x0F11



/* Notes: The application may submit a Call Clear request before receiving   */
/*  a Confirm to a prior Call Establish Request. Since in this case the      */
/*  Application does not yet have a P_CI_ID, it will set the IDType parameter*/
/*  to CMI_CM_ID_TYPE_P_SAP_ID, and put the P_SAP_ID into the Identifier     */
/*  parameter. In this case the application must also set the U_CI_ID        */
/*  parameter  to the Call Instance being cleared.                           */



/*****************************************************************************/
/*                                                                           */
/*                   CM_CALL_CLEAR Confirm Primitive                         */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_CLEAR Confirm
// ---------------------------------------------
typedef struct CI_CM_Call_Clear_Cnf
{
     WORD               CtrlInfoLength; // sizeof(CI_CM_CALL_CLEAR_CNF)
     WORD               PrimitiveCode;  // CMI_CM_PC_CALL_CLEAR_CNF
     DWORD              DataInfoLength; // length of Data Information Area
     WORD               Selector;       // 0
     WORD               IDType;         // CMI_CM_ID_TYPE_U_CI_ID
     DWORD              U_CI_ID;        // Appl assign Call id
     CMI_COM_COMSTATUS  CommonStatus;   //
} CI_CM_CALL_CLEAR_CNF;

// Primitive Code for CM_CALL_CLEAR Confirm
// ----------------------------------------
#define CMI_CM_PC_CALL_CLEAR_CNF        0xCF11



/*****************************************************************************/
/*                                                                           */
/*                  CM_CALL_CLEAR Indication Primitive                       */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_CLEAR Indication
// ------------------------------------------------
typedef struct CI_CM_Call_Clear_Ind
{
     WORD               CtrlInfoLength; // sizeof(CI_CM_CALL_CLEAR_IND)
     WORD               PrimitiveCode;  // CMI_CM_PC_CALL_CLEAR_IND
     DWORD              DataInfoLength; // length of Data Information Area
     WORD               Selector;       // 0
     WORD               IDType;         // CMI_CM_ID_TYPE_U_CI_ID OR
                                        // CMI_CM_ID_TYPE_U_SAP_ID
     DWORD              Identifier;     //
     CMI_COM_COMSTATUS  CommonStatus;   //
     DWORD              P_CI_ID;        //
     DWORD              Pad_1;          // Alignment
} CI_CM_CALL_CLEAR_IND;

// Primitive Code for CM_CALL_CLEAR Indication
// -------------------------------------------
#define CMI_CM_PC_CALL_CLEAR_IND        0x4F11


/* Notes: The application may receive a Call Clear indicate before it can    */
/*  send a Call Establish response to a prior Call Establish Indicate.       */
/*  Since in this case CM does not yet have a U_CI_ID, it will set the IDType*/
/*  parameter to CMI_CM_ID_TYPE_U_SAP_ID, and put the U_SAP_ID into the      */
/*  Identifier parameter. In this case CM will also set the P_CI_ID          */
/*  parameter to the Call Instance being cleared.                            */



/*****************************************************************************/
/*                  PCM_CALL_CLEAR Response Primitive                        */
/*****************************************************************************/

// Control Information for CM_CALL_CLEAR Response
// ----------------------------------------------
typedef struct CI_CM_Call_Clear_Rsp
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_CALL_CLEAR_RSP)
     WORD   PrimitiveCode;  // CMI_CM_PC_CALL_CLEAR_RSP
     DWORD  DataInfoLength; // length of Data Information Area
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_P_CI_ID
     DWORD  P_CI_ID;        // Provider assigned Call Instance Id
} CI_CM_CALL_CLEAR_RSP;

// Primitive Code for PCM_CALL_CLEAR Response
// ------------------------------------------
#define CMI_CM_PC_CALL_CLEAR_RSP        0x8F11



/*****************************************************************************/
/*                                                                           */
/*                CM_CALL_CONNECTED Indication Primitive                     */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_CONNECTED Indication
// ----------------------------------------------------
typedef struct CI_CM_Call_Connected_Ind
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_CALL_CONNECTED_IND)
     WORD   PrimitiveCode;  // CMI_CM_PC_CALL_CONNECTED_IND
     DWORD  DataInfoLength; // length of Data Information Area
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_U_CI_ID
     DWORD  U_CI_ID;        // Appl assigned Call Id
} CI_CM_CALL_CONNECTED_IND;

// Primitive Code for CM_CALL_CONNECTED Indication
// -----------------------------------------------
#define CMI_CM_PC_CALL_CONNECTED_IND       0x4F12


/*****************************************************************************/
/*                                                                           */
/*                CM_ENABLE_INCOMING_CALLS Request Primitive                 */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_ENABLE_INCOMING Request
// --------------------------------------------------
typedef struct CI_CM_Enable_Incoming_Req
{
     WORD   CtrlInfoLength;  // sizeof(CI_CM_ENABLE_INCOMING_REQ)
     WORD   PrimitiveCode;   // CMI_CM_PC_ENABLE_INCOMING_REQ
     DWORD  DataInfoLength;  // length of Data Information Area
     WORD   Selector;        // 0
     WORD   IDType;          // CMI_CM_ID_TYPE_P_SAP_ID
     DWORD  P_SAP_ID;        // CM assigned SAP identifier
     DWORD  U_CE_ID;         // Appl assigned Call Enablment id
     DWORD  Pad_1;           // Alignment
} CI_CM_ENABLE_INCOMING_REQ;

// Primitive Code for CM_ENABLE_INCOMING Request
// ---------------------------------------------
#define CMI_CM_PC_ENABLE_INCOMING_REQ   0x0FA0


// Primitive Specific Subfield for CM_ENABLE_INCOMING Request
// ----------------------------------------------------------
typedef struct SF_CM_Enable_Incoming_Req
{
     WORD   SFLength;       // sizeof(SF_CM_ENABLE_INCOMING_REQ)
     BYTE   SFIdentifier;   // CMI_CM_SFID_PRM_SPECIFIC
     BYTE   SFType;         // CMI_CM_SFTYPE_PRM_SPECIFIC
     BYTE   SFEntryName[16];// Incoming Call Directory Entry
     WORD   SFUsageInd;     //
     WORD   SFPad_1;        // Alignment
     BYTE   SFPcmName[16];  // Enable Calls thru from this PCM
     WORD   SFPcmType;      // Enable Calls thru this PCM type
     WORD   SFPad_2;        // Alignment
} SF_CM_ENABLE_INCOMING_REQ;

// Usage Indicator
// ---------------
#define CMI_CM_USAGE_MUST_MATCH         0x00
#define CMI_CM_USAGE_MUST_NOT_MATCH     0x01

// PCM type
// --------
#define CMI_CM_PCM_TYPE_UNSPECIFIED     0x10
#define CMI_CM_PCM_TYPE_ISDN            0x01



/*****************************************************************************/
/*                                                                           */
/*                CM_ENABLE_INCOMING_CALLS Confirm Primitive                 */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_ENABLE_INCOMING Confirm
// --------------------------------------------------
typedef struct CI_CM_Enable_Incoming_Cnf
{
     WORD               CtrlInfoLength; // sizeof(CI_CM_ENABLE_INCOMING_CNF)
     WORD               PrimitiveCode;  // CMI_CM_PC_ENABLE_INCOMING_CNF
     DWORD              DataInfoLength; // 0
     WORD               Selector;       // 0
     WORD               IDType;         // CMI_CM_ID_TYPE_U_CE_ID
     DWORD              U_CE_ID;        // Appl assigned Call Enablement id
     CMI_COM_COMSTATUS  CommonStatus;   //
     DWORD              P_CE_ID;        // CM assigned Call Enablement id
     DWORD              Pad_1;          // Alignment
} CI_CM_ENABLE_INCOMING_CNF;

// Primitive Code for CM_ENABLE_INCOMING Confirm
// ---------------------------------------------
#define CMI_CM_PC_ENABLE_INCOMING_CNF   0xCFA0



/*****************************************************************************/
/*                                                                           */
/*                CM_DISABLE_INCOMING_CALLS Request Primitive                */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_DISABLE_INCOMING Request
// ---------------------------------------------------
typedef struct CI_CM_Disable_Incoming_Req
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_DISABLE_INCOMING_REQ)
     WORD   PrimitiveCode;  // CMI_CM_PC_DISABLE_INCOMING_REQ
     DWORD  DataInfoLength; // 0
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_P_CE_ID
     DWORD  P_CE_ID;        // CM assigned Call Enablement id
} CI_CM_DISABLE_INCOMING_REQ;

// Primitive Code for CM_DISABLE_INCOMING Request
// ----------------------------------------------
#define CMI_CM_PC_DISABLE_INCOMING_REQ  0x0FA1



/*****************************************************************************/
/*                                                                           */
/*                CM_DISABLE_INCOMING_CALLS Confirm Primitive                */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_DISABLE_INCOMING Confirm
// ---------------------------------------------------
typedef struct CI_CM_Disable_Incoming_Cnf
{
     WORD               CtrlInfoLength; // sizeof(CI_CM_DISABLE_INCOMING_CNF)
     WORD               PrimitiveCode;  // CMI_CM_PC_DISABLE_INCOMING_CNF
     DWORD              DataInfoLength; // 0
     WORD               Selector;       // 0
     WORD               IDType;         // CMI_CM_ID_TYPE_U_CE_ID
     DWORD              U_CE_ID;        // Appl assigned Call Enablement id
     CMI_COM_COMSTATUS  CommonStatus;   //  added
} CI_CM_DISABLE_INCOMING_CNF;

// Primitive Code for CM_DISABLE_INCOMING Confirm
// ----------------------------------------------
#define CMI_CM_PC_DISABLE_INCOMING_CNF  0xCFA1



/*****************************************************************************/
/*                                                                           */
/*                   CM_CALL_CONTROL Request Primitive                       */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_CONTROL Request
// -----------------------------------------------
typedef struct CI_CM_Call_Ctrl_Req
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_CALL_CTRL_REQ)
     WORD   PrimitiveCode;  // CMI_CM_PC_CALL_CTRL_REQ
     DWORD  DataInfoLength; // Length of Data Information Area
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_P_CI_ID
     DWORD  P_CI_ID;        // CM assigned Call Id
} CI_CM_CALL_CTRL_REQ;

// Primitive Code for CM_CALL_CONTROL Request
// ------------------------------------------
#define CMI_CM_PC_CALL_CTRL_REQ         0x0F1A



/*****************************************************************************/
/*                                                                           */
/*                   CM_CALL_CONTROL Confirm Primitive                       */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_CONTROL Confirm
// -----------------------------------------------
typedef struct CI_CM_Call_Ctrl_Cnf
{
     WORD               CtrlInfoLength; // sizeof(CI_CM_CALL_CTRL_CNF)
     WORD               PrimitiveCode;  // CMI_CM_PC_CALL_CTRL_CNF
     DWORD              DataInfoLength; // 0
     WORD               Selector;       // 0
     WORD               IDType;         // CMI_CM_ID_TYPE_U_CI_ID
     DWORD              U_CI_ID;        // Appl assigned Call Id
     CMI_COM_COMSTATUS  CommonStatus;
} CI_CM_CALL_CTRL_CNF;

// Primitive Code for CM_CALL_CONTROL Confirm
// ------------------------------------------
#define CMI_CM_PC_CALL_CTRL_CNF        0xCF1A



/*****************************************************************************/
/*                                                                           */
/*                  CM_CALL_STATUS Indication Primitive                      */
/*                                                                           */
/*****************************************************************************/

// Control Information for CM_CALL_STATUS Indication
// -------------------------------------------------
typedef struct CI_CM_Call_Stat_Ind
{
     WORD   CtrlInfoLength; // sizeof(CI_CM_CALL_STAT_IND)
     WORD   PrimitiveCode;  // CMI_CM_PC_CALL_STAT_IND
     DWORD  DataInfoLength; // length of Data Information Area
     WORD   Selector;       // 0
     WORD   IDType;         // CMI_CM_ID_TYPE_U_CI_ID
     DWORD  U_CI_ID;        // Appl assigned Call Id
} CI_CM_CALL_STAT_IND;

// Primitive Code for CM_CALL_STATUS Indication
// --------------------------------------------
#define CMI_CM_PC_CALL_STAT_IND         0x4F14






/*****************************************************************************/
/*                                                                           */
/*             Primitve Specific Subfield OverLay                            */
/*                                                                           */
/*****************************************************************************/
#ifndef _SF_HEADER_32
typedef struct SF_Header
{
   WORD  SFLength;         // sizeof(SF_HEADER)-1+subfield data
   BYTE  SFIdentifier;     // SF_SFID_*
   BYTE  SFType;           // SF_*_TYPE_**
} SF_HEADER;
#define _SF_HEADER_32
#endif


#ifndef _CI_HEADER_32
typedef struct ci_header
{
   WORD   CtrlInfoLength;  // Control info length (inclusive)
   WORD   PrimitiveCode;   // Control ID code
   DWORD  DataInfoLength;  // Length of data in control block
} CI_HEADER;
#define _CI_HEADER_32
#endif

typedef union
{
   /* Generic Primitives */
   CI_HEADER                     Header;

   /* Primitives */
   CI_CM_ACT_SAP_REQ             ActSAPReq;
   CI_CM_ACT_SAP_CNF             ActSAPCnf;
   CI_CM_DEACT_SAP_REQ           DeactSAPReq;
   CI_CM_DEACT_SAP_CNF           DeactSAPCnf;
   CI_CM_DEACT_SAP_IND           DeactSAPInd;
   CI_CM_CALL_ESTAB_REQ          CallEstabReq;
   CI_CM_CALL_ESTAB_CNF          CallEstabCnf;
   CI_CM_CALL_ESTAB_IND          CallEstabInd;
   CI_CM_CALL_ESTAB_RSP          CallEstabRsp;
   CI_CM_CALL_CLEAR_REQ          CallClearReq;
   CI_CM_CALL_CLEAR_CNF          CallClearCnf;
   CI_CM_CALL_CLEAR_IND          CallClearInd;
   CI_CM_CALL_CLEAR_RSP          CallClearRsp;
   CI_CM_CALL_CONNECTED_IND      CallConnectedInd;
   CI_CM_ENABLE_INCOMING_REQ     EnableIncomingReq;
   CI_CM_ENABLE_INCOMING_CNF     EnableIncomingCnf;
   CI_CM_DISABLE_INCOMING_REQ    DisableIncomingReq;
   CI_CM_DISABLE_INCOMING_CNF    DisableIncomingCnf;
   CI_CM_CALL_CTRL_CNF           CallCtrlCnf;
   CI_CM_CALL_STAT_IND           CallStatInd;
} CI_PRIMITIVE;

typedef union
{
   /* Generic Subfields */
   SF_HEADER                    Header;

   /* Primitive Specific Subfields */
   SF_CM_ACT_SAP_CNF            ActSAPCnf;
   SF_CM_CALL_ESTAB_REQ         CallEstabReq;
   SF_CM_CALL_ESTAB_CNF         CallEstabCnf;
   SF_CM_CALL_ESTAB_IND         CallEstabInd;
   SF_CM_MON_CALL_ESTAB_IND     MonEstabInd;
   SF_CM_CALL_ESTAB_RSP         CallEstabRsp;
   SF_CM_ENABLE_INCOMING_REQ    EnableIncomingReq;

} PRIM_SUBFIELD;

typedef union
{
   SF_CM_ACT_SAP_CNF            *pActSAPCnf;
   SF_CM_CALL_ESTAB_REQ         *pCallEstabReq;
   SF_CM_CALL_ESTAB_CNF         *pCallEstabCnf;
   SF_CM_CALL_ESTAB_IND         *pCallEstabInd;
   SF_CM_MON_CALL_ESTAB_IND     *pMonEstabInd;
   SF_CM_CALL_ESTAB_RSP         *pCallEstabRsp;
   SF_CM_ENABLE_INCOMING_REQ    *pEnableIncReq;

} P_CM_SUBFIELD;

#endif
